/****************************************************************************
**
** Copyright (C) 2017 The Qt Company Ltd.
** Contact: http://www.qt.io/licensing/
**
** This file is part of the Qt Quick Controls 2 module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL3$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see http://www.qt.io/terms-conditions. For further
** information use the contact form at http://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPLv3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or later as published by the Free
** Software Foundation and appearing in the file LICENSE.GPL included in
** the packaging of this file. Please review the following information to
** ensure the GNU General Public License version 2.0 requirements will be
** met: http://www.gnu.org/licenses/gpl-2.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

import QtQuick 2.12
import QtQuick.Templates 2.12 as T
import QtQuick.Controls 2.12
import QtQuick.Controls.impl 2.12
import org.ukui.qqc2style.private 1.0 as StylePrivate

T.SwitchDelegate {
    id: control

    implicitWidth: Math.max(implicitBackgroundWidth + leftInset + rightInset,
                            implicitContentWidth + leftPadding + rightPadding)
    implicitHeight: Math.max(implicitBackgroundHeight + topInset + bottomInset,
                             implicitContentHeight + topPadding + bottomPadding,
                             implicitIndicatorHeight + topPadding + bottomPadding)

    padding: ukuiSwitch.padding
    spacing: 12

//    icon.width: 24
//    icon.height: 24
//    icon.color: control.palette.text

    StylePrivate.UKUISwitch{
    id: ukuiSwitch
    }
    ParseInterface{
        id: parseInterface
    }

    function getStartColor(dtcolor)
    {
        return parseInterface.pStartColor(dtcolor)
    }
    function getEndColor(dtcolor)
    {
        return parseInterface.pEndColor(dtcolor)
    }

    indicator: Rectangle {
        implicitWidth: ukuiSwitch.recWidth
        implicitHeight: ukuiSwitch.recHeight

        x: control.text ? (control.mirrored ? control.leftPadding : control.width - width - control.rightPadding) : control.leftPadding + (control.availableWidth - width) / 2
        y: control.topPadding + (control.availableHeight - height) / 2

        radius: implicitHeight/2
//        leftPadding: 0
//        rightPadding: 0
//        padding: (height - 16) / 2
        color: getStartColor(control.checked ? (!control.enabled ? ukuiSwitch.checkedDisableColor : control.pressed ? ukuiSwitch.checkedClickColor : control.hovered ? ukuiSwitch.checkedHoverColor : ukuiSwitch.checkedNormalColor)
                               : (!control.enabled ? ukuiSwitch.disableColor : control.pressed ? ukuiSwitch.clickColor : control.hovered ? ukuiSwitch.hoverColor : ukuiSwitch.normalColor))

        border.color: getStartColor(control.checked ? (!control.enabled ? ukuiSwitch.checkedDisableBorderColor : control.pressed ? ukuiSwitch.checkedClickBorderColor : control.hovered ? ukuiSwitch.checkedHoverBorderColor : ukuiSwitch.checkedNormalBorderColor)
                               : (!control.enabled ? ukuiSwitch.disableBorderColor : control.pressed ? ukuiSwitch.clickBorderColor : control.hovered ? ukuiSwitch.hoverColor : ukuiSwitch.normalBorderColor))

        border.width: ukuiSwitch.borderWidth
        Rectangle {
            x: Math.max(ukuiSwitch.indicatorLeftRightPadding, Math.min(parent.width - width - ukuiSwitch.indicatorLeftRightPadding, control.visualPosition * parent.width - (width / 2) - ukuiSwitch.indicatorLeftRightPadding))
            y: (parent.height - height) / 2
            width: ukuiSwitch.indicatorWidth
            height: ukuiSwitch.indicatorWidth
            radius: height/2
            color: getStartColor(!control.enabled ? ukuiSwitch.disableIndicatorColor : ukuiSwitch.normalIndicatorColor)
//            border.width: control.visualFocus ? 2 : 1
//            border.color: control.visualFocus ? control.palette.highlight : control.enabled ? control.palette.mid : control.palette.midlight

            Behavior on x {
                enabled: !control.down
                SmoothedAnimation { velocity: 200 }
            }
        }
    }

    contentItem: IconLabelContent {
        id: iconLabel

        controlRoot: control

        anchors.left: control.left
        leftSpace: control.mirrored ? control.indicator.width + control.spacing : control.leftPadding

        implicitWidth: iconLabel.adjustWidth(control.leftPadding + control.rightPadding) + indicator.implicitWidth + control.spacing
    }

    background: Rectangle {
        implicitWidth: parent.implicitWidth
        implicitHeight: parent.implicitHeight
        visible: control.down || control.highlighted
        color: control.down ? control.palette.midlight : control.palette.light
    }
}
